#!/bin/ksh
# LED - Log Event Distribution

#p="\] OK LOGIN"
#log="/var/log/vsftpd.log"

p=$(echo ${*:4})
type="${3}"
log="${2}"
freq="${1}"

p=$(echo ${p} | sed -r 's/(\[|\])/\\&/g')

configure() {
	datenow=$(date +'%s')
	if [ ! -r "${log}" ] ; then echo "Log file ${log} not found" ; exit 1 ; fi
}

# prints sum, count, average, median, min datum, max datum, etc.
avg() {
		sort -n | awk '
	  BEGIN {
		c = 0;
		sum = 0;
	  }
	  $1 ~ /^[0-9]*(\.[0-9]*)?$/ {
		a[c++] = $1;
		sum += $1;
	  }
	  END {
		ave = sum / c;
		printf("%.0f\n",ave);
	  }'
}

min() {
	sort -n | awk '
	  BEGIN {
		c = 0;
		sum = 0;
	  }
	  $1 ~ /^[0-9]*(\.[0-9]*)?$/ {
		a[c++] = $1;
		sum += $1;
	  }
	  END {
		c=asort(a);
		print a[1];
	  }'
}

max() {
	sort -n | awk '
	  BEGIN {
		c = 0;
		sum = 0;
	  }
	  $1 ~ /^[0-9]*(\.[0-9]*)?$/ {
		a[c++] = $1;
		sum += $1;
	  }
	  END {
	    c=asort(a);
		print a[c];
	  }'
}

median() {
	sort -n | awk '
	  BEGIN {
		c = 0;
		sum = 0;
	  }
	  $1 ~ /^[0-9]*(\.[0-9]*)?$/ {
		a[c++] = $1;
		sum += $1;
	  }
	  END {
		ave = sum / c;
		if( (c % 2) == 1 ) {
		  median = a[ int(c/2) ];
		} else {
		  median = ( a[c/2] + a[c/2-1] ) / 2;
		}
		OFS="\t";
		print median;
	  }'
}

sum() {
	sort -n | awk '
	  BEGIN {
		c = 0;
		sum = 0;
	  }
	  $1 ~ /^[0-9]*(\.[0-9]*)?$/ {
		a[c++] = $1;
		sum += $1;
	  }
	  END {
		print sum;
	  }'
}

count() {
	sort -n | awk '
	  BEGIN {
		c = 0;
		sum = 0;
	  }
	  $1 ~ /^[0-9]*(\.[0-9]*)?$/ {
		a[c++] = $1;
		sum += $1;
	  }
	  END {
		print c;
	  }'
}

maxdelta() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`"|bc -l)
		echo ${array_d[$i]}
	done | max
}

mindelta() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`"|bc -l)
		echo ${array_d[$i]}
	done | min
}

avgdelta() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`"|bc -l)
		echo ${array_d[$i]}
	done | avg
}

avgdelta_s() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`"|bc -l)
		echo ${array_d[$i]}
	done | avg
}

avgdelta_m() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;(`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`)/60"|bc -l)
		echo ${array_d[$i]}
	done | avg
}

avgdelta_h() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;(`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`)/3600"|bc -l)
		echo ${array_d[$i]}
	done | avg
}

avgdelta_d() {
	i=0 ; unset array_a ; unset array_d ; while read line ; do array_a[$i]="$line" ; (( i++ )) ; done
	array_ac=$(echo "scale=0;`echo ${#array_a[*]}`-2"|bc -l)
	for i in `seq 0 $array_ac`
	do
		array_d[$i]=$(echo "scale=0;(`echo ${array_a[(( i + 1 ))]}`-`echo ${array_a[i]}`)/86400"|bc -l)
		echo ${array_d[$i]}
	done | avg
}

timediff_s() {
i=0 ; unset array_a ; unset array_d ; tee >(min) >(max) 1>/dev/null | while read line
	do
		array_a[$i]="$line"
		if [ ${i} -eq 1 ]
			then
			echo "scale=0;`echo ${array_a[i]}`-`echo ${array_a[0]}`"|bc -l
		fi
		(( i++ ))
	done
}

timediff_m() {
i=0 ; unset array_a ; unset array_d ; tee >(min) >(max) 1>/dev/null | while read line
	do
		array_a[$i]="$line"
		if [ ${i} -eq 1 ]
			then
			echo "scale=0;(`echo ${array_a[i]}`-`echo ${array_a[0]}`)/60"|bc -l
		fi
		(( i++ ))
	done
}

timediff_h() {
	i=0 ; unset array_a ; unset array_d ; tee >(min) >(max) 1>/dev/null | while read line
	do
		array_a[$i]="$line"
		if [ ${i} -eq 1 ]
			then
			echo "scale=0;(`echo ${array_a[i]}`-`echo ${array_a[0]}`)/3600"|bc -l
		fi
		(( i++ ))
	done
}

timediff_d() {
	i=0 ; unset array_a ; unset array_d ; tee >(min) >(max) 1>/dev/null | while read line
	do
		array_a[$i]="$line"
		if [ ${i} -eq 1 ]
			then
			echo "scale=0;(`echo ${array_a[i]}`-`echo ${array_a[0]}`)/86400"|bc -l
		fi
		(( i++ ))
	done
}

buildarray_m() {
	i=0
	case ${type} in
		1) fields="2" ;;
		2) fields="1" ;;
		*) exit 1 ;;
	esac
	zgrep -E "${p}" "${log}"* | tr -s ' ' | cut -d: -f2- | cut -d' ' -f${fields} | sort -k1M -u | while read line
	do
		c=$(zgrep -E "${p}" "${log}"* | tr -s ' ' | cut -d: -f2- | grep -c "${line}\b")
		e=$(date --date="${line} 1" +'%s')
		if [ ${e} -gt ${datenow} ]
		then
			d=$(date --date="${line} 1 year ago" +'%Y-%b')
		else
			d=$(date --date="${line} 1" +'%Y-%b')
		fi
		a1[$i]="${d}"
		a2[$i]="${c}"
		(( i = i + 1 ))
	done
}

buildarray_d() {
	i=0
	case ${type} in
		1) fields="2-3" ;;
		2) fields="1-2" ;;
		*) exit 1 ;;
	esac
	zgrep -E "${p}" "${log}"* | tr -s ' ' | cut -d: -f2- | cut -d' ' -f${fields} | sort -k1M -k2n -u | while read line
	do
		c=$(zgrep -E "${p}" "${log}"* | tr -s ' ' | cut -d: -f2- | grep -c "${line}\b")
		e=$(date --date="${line}" +'%s')
		if [ ${e} -gt ${datenow} ]
		then
			d=$(date --date="${line} 1 year ago" +'%Y-%m-%d %a')
		else
			d=$(date --date="${line}" +'%Y-%m-%d %a')
		fi
		a1[$i]="${d}"
		a2[$i]="${c}"
		(( i = i + 1 ))
	done
}

buildarray_h() {
	i=0
	case ${type} in
		1) fields="2-4" ;;
		2) fields="1-3" ;;
		*) exit 1 ;;
	esac
	zgrep -E "${p}" "${log}"* | tr -s ' ' | cut -d: -f2- | cut -d: -f1 | cut -d' ' -f${fields} | sort -k1M -k2n -k3n -u | while read line
	do
		c=$(zgrep -E "${p}" "${log}"* | tr -s ' ' | cut -d: -f2- | grep -c "${line}\b")
		e=$(date --date="${line}" +'%s')
		if [ ${e} -gt ${datenow} ]
		then
			d=$(date --date="${line} 1 year ago" +'%Y-%m-%d %H %p')
		else
			d=$(date --date="${line}" +'%Y-%m-%d %H %p')
		fi
		a1[$i]="${d}"
		a2[$i]="${c}"
		(( i = i + 1 ))
	done
}

makepretty() {
	w=60
	max=$(printf '%s\n' "${a2[@]}" | sort -nr | head -n1)
	if [ "${max}" ] && [ ${max} -gt 0 ]
	then
		x=$(echo "scale=4;${w}/${max}"|bc -l)
		for i in $(seq 0 `echo "scale=0;${#a1[*]}-1"|bc -l`)
		do
			echo -ne "${a1[$i]}\t${a2[$i]}\t"
			for j in $(seq 0 `echo "scale=0;$(echo ${a2[$i]})*${x}/1"|bc -l`)
			do
				printf '\033[%s;%s;%sm %0s' 5 30 43 . | sed 's/ //g'
			done
			printf '\033[%s;%s;%sm %0s \033[0m \n' 5 30 43 ${a2[$i]}
		done
	else
		echo "No matches"
	fi
}

# RUNTIME

configure

case $freq in
	m) buildarray_m ;;
	d) buildarray_d ;;
	h) buildarray_h ;;
	*) exit 1 ;;
esac

makepretty